<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Renderer;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Returns responses for Simple Interactive Maps routes.
 */
final class AjaxMapLoaderController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    private MapDataLoader $simpleInteractiveMapsMapDataLoader,
    private Renderer $renderer,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('simple_interactive_maps.map_data_loader'),
      $container->get('renderer'),
    );
  }

  /**
   * Builds the response.
   */
  public function __invoke(InteractiveMapInterface $interactive_map = NULL, string $target_element): AjaxResponse {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(404, 'Interactive map not found.');
    }

    $mapData = $this->simpleInteractiveMapsMapDataLoader->loadMapData($interactive_map);

    $build['map'] = [
      '#theme' => 'interactive_map',
      '#map' => $interactive_map,
    ];

    $rendered_map = $this->renderer->render($build);

    $response = new AjaxResponse();
    $ajax_command = new ReplaceCommand('#' . $target_element, (string) $rendered_map);
    $response->addCommand($ajax_command);
    return $response;
  }

}
